#include "main.h"

GList *nexxus_list = NULL;

Nexxus *
flim_nexxus_new(void)
{
   Nexxus *nx;
   
   nx = g_malloc(sizeof(Nexxus));
   nx->type = 0;
   nx->connection = NULL;
   nx->name = NULL;
   nx->address = NULL;
   nx->nodes = NULL;
   nx->login = g_strdup("blum");
   nx->password = g_strdup("frub");
   nx->selected = 0;
   return nx;
}

Nexxus *
flim_nexxus_find(char *name)
{
   GList *l;
   
   for (l = nexxus_list; l; l = l->next)
     {
	Nexxus *nx;
	
	nx = l->data;
	if (!strcmp(nx->name, name))
	   return nx;
     }
   return NULL;
}

void
flim_nexxus_free(Nexxus *nx)
{
   flim_nexxus_disconnect(nx);
   if (nx->address)
      g_free(nx->address);
   if (nx->name)
      g_free(nx->name);
   if (nx->login)
      g_free(nx->login);
   if (nx->password)
      g_free(nx->password);
   while (nx->nodes) flim_nexxus_del_node(nx, nx->nodes->data);
   g_free(nx);
}

void
flim_nexxus_add(Nexxus *nx)
{
   Msg msg;
   char *name, *address, *login, *password;
   
   nexxus_list = g_list_append(nexxus_list, nx);
   g_snprintf(msg.msg, sizeof(msg.msg), "nexxus_add");
   name = nx->name;
   address = nx->address;
   login = nx->login;
   password = nx->password;
   if (!name) name = "";
   if (!address) address = "";
   if (!login) login = "";
   if (!password) password = "";   
   g_snprintf(msg.dat, sizeof(msg.dat), "%s:%s:%s:%s", name, address, login, password);
   flim_plugin_broadcast(&msg);
}

void
flim_nexxus_del(Nexxus *nx)
{
   Msg msg;
   
   nexxus_list = g_list_remove(nexxus_list, nx);
   g_snprintf(msg.msg, sizeof(msg.msg), "nexxus_del");
   g_snprintf(msg.dat, sizeof(msg.dat), "%s", nx->name);
   flim_plugin_broadcast(&msg);
}

void
flim_nexxus_disconnect(Nexxus *nx)
{
   while (nx->nodes)
      flim_nexxus_del_node(nx, nx->nodes->data);
   if (nx->connection)
      flim_comms_disconnect_nexxus(nx->connection);
   nx->connection = NULL;
}

void
flim_nexxus_connect(Nexxus *nx)
{
   char error_ret[4096];
   
   nx->connection = flim_comms_connect_nexxus(nx->address, error_ret, nx->login, nx->password);
   if (!nx->connection)
     {
	message(error_ret);
	return;
     }
   FLIM_COMMS_BEGIN(nx->connection, "nexxus:node_list");
   while (FLIM_COMMS_REPLY)
     {
	Node *nd;
	
	if (FLIM_COMMS_REPLY_COUNT < 3)
	   break;
	if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
	   break;
        if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
	   break;
	nd = flim_node_new();
/*	printf("%s %s %s %s %s\n", 
	       FLIM_COMMS_REPLY[0],
	       FLIM_COMMS_REPLY[1],
	       FLIM_COMMS_REPLY[2],
	       FLIM_COMMS_REPLY[3],
	       FLIM_COMMS_REPLY[4]);*/
	if (!strcmp("NODELIST", FLIM_COMMS_REPLY[2]))
	   nd->name = g_strdup(FLIM_COMMS_REPLY[4]);
	flim_nexxus_add_node(nx, nd);
	FILM_COMMS_GET_REPLY;
     }
   FILM_COMMS_REPLY_END;
     {  
	GList *l;
	
	for (l = nx->nodes; l; l = l->next)
	  {
	     Node *nd;
	     
	     nd = l->data;
	     flim_node_get_info(nx, nd);
	  }
     }
}

void
flim_nexxus_add_node(Nexxus *nx, Node *nd)
{
   Msg msg;
   
   nd->nexxus = nx;
   nx->nodes = g_list_append(nx->nodes, nd);
   g_snprintf(msg.msg, sizeof(msg.msg), "node_add");
   g_snprintf(msg.dat, sizeof(msg.dat), "%s:%s", nd->nexxus->name, nd->name);
   flim_plugin_broadcast(&msg);
}

void
flim_nexxus_del_node(Nexxus *nx, Node *nd)
{
   Msg msg;
   
   g_snprintf(msg.msg, sizeof(msg.msg), "node_del");
   g_snprintf(msg.dat, sizeof(msg.dat), "%s:%s", nd->nexxus->name, nd->name);
   nd->nexxus = NULL;
   nx->nodes = g_list_remove(nx->nodes, nd);
   flim_plugin_broadcast(&msg);
}

void
flim_nexxus_load_config(void)
{
   FILE *f;
   gchar *home, buf[4096];
   Nexxus *nx = NULL;
   int     next_nexxus = 0, next_address = 0, next_login = 0, next_password = 0;

   home = getenv("HOME");
   if (!home) return;
   g_snprintf(buf, sizeof(buf), "%s/.flim/nexxus", home);
   f = fopen(buf, "r");
   if (!f)
      return;
   while (fgets(buf, sizeof(buf) - 1, f))
     {
	char s1[4096], s2[4096];
	
	if (strlen(buf) > 0)
	   buf[strlen(buf) - 1] = 0;
	sscanf(buf, "%s %s", s1, s2);
	if (next_nexxus)
	  {
	     if (!nx)
		nx = flim_nexxus_new();
	     nx->name = g_strdup(buf);
	     next_nexxus = 0;
	     next_address = 1;
	  }
	else if (next_address)
	  {
	     if (!nx)
		nx = flim_nexxus_new();
	     nx->address = g_strdup(buf);
	     next_address = 0;
	     next_login = 1;
	  }
	else if (next_login)
	  {
	     if (!nx)
		nx = flim_nexxus_new();
	     nx->login = g_strdup(buf);
	     next_login = 0;
	     next_password = 1;
	  }
	else if (next_password)
	  {
	     if (!nx)
		nx = flim_nexxus_new();
	     nx->password = g_strdup(buf);
	     flim_nexxus_add(nx);
	     flim_nexxus_connect(nx);
	     next_password = 0;
	     nx = NULL;
	  }
        else if (!strcmp("::::", s1))
	  {
	     if (!strcmp("nexxus", s2)) next_nexxus = 1;
	  }
     }
   fclose(f);
}

void
flim_nexxus_save_config(void)
{
   FILE *f;
   GList *l;
   gchar *home, buf[4096];
   
   home = getenv("HOME");
   if (!home) return;
   g_snprintf(buf, sizeof(buf), "%s/.flim", home);
   mkdir(buf, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
   g_snprintf(buf, sizeof(buf), "%s/.flim/nexxus", home);
   f = fopen(buf, "w");
   if (!f)
      return;
   for (l = nexxus_list; l; l = l->next)
     {
	Nexxus *nx;
	
	nx = l->data;
	fprintf(f, ":::: nexxus\n");
	fprintf(f, "%s\n", nx->name);
	fprintf(f, "%s\n", nx->address);
	if (nx->login)
	   fprintf(f, "%s\n", nx->login);
	else
	   fprintf(f, "\n");
	if (nx->password)
	   fprintf(f, "%s\n", nx->password);
	else
	   fprintf(f, "\n");
     }
   fclose(f);
}

GtkWidget *pm_good = NULL, *pm_bad, *pm_warning, *pm_nexxus;

void
flim_nexxus_build_tree(GtkWidget *tree)
{
   GList *l;

   /* if pixmap havent been created - make them */
   if (!pm_good)
     {
	pm_good = create_pixmap(tree, "node_good.xpm");
	pm_bad = create_pixmap(tree, "node_bad.xpm");
	pm_warning = create_pixmap(tree, "node_warning.xpm");
	pm_nexxus = create_pixmap(tree, "nexxus.xpm");
     }
   /* start - freeze & clear tree */
   gtk_clist_freeze(GTK_CLIST(tree));
   gtk_clist_clear(GTK_CLIST(tree));
   gtk_clist_set_row_height(GTK_CLIST(tree), 18);
   gtk_clist_set_column_width    (GTK_CLIST(tree), 0, 10);
   gtk_clist_set_column_min_width(GTK_CLIST(tree), 0, 10);
   gtk_clist_set_column_max_width(GTK_CLIST(tree), 0, 10);
   gtk_clist_set_column_width    (GTK_CLIST(tree), 1, 16);
   gtk_clist_set_column_min_width(GTK_CLIST(tree), 1, 16);
   gtk_clist_set_column_max_width(GTK_CLIST(tree), 1, 16);
   gtk_clist_set_column_width    (GTK_CLIST(tree), 2, 70);
   gtk_clist_set_column_min_width(GTK_CLIST(tree), 2, 70);
   gtk_clist_set_column_max_width(GTK_CLIST(tree), 2, 70);
   gtk_ctree_set_indent(GTK_CTREE(tree), 0);
   gtk_ctree_set_line_style(GTK_CTREE(tree), GTK_CTREE_LINES_DOTTED);
   gtk_ctree_set_expander_style(GTK_CTREE(tree), GTK_CTREE_EXPANDER_CIRCULAR);
   for (l = nexxus_list; l; l = l->next)
     {
	GList *ll;
	GtkCTreeNode *ctree_node;
	Nexxus *nx;
	gchar *text[3];
	GdkPixmap *p, *m;
	
	nx = l->data;
	nx->selected = 0;
	/* add parent branch for nexxus */
	text[0] = ""; text[1] = "", text[2] = nx->name;
	ctree_node = gtk_ctree_insert_node(GTK_CTREE(tree), NULL, NULL,
					   text, 1, NULL, NULL, NULL, 
					   NULL, FALSE, TRUE);
	gtk_ctree_node_set_row_data(GTK_CTREE(tree), ctree_node, nx);
	gtk_pixmap_get(GTK_PIXMAP(pm_nexxus), &p, &m);
	gtk_ctree_node_set_pixmap(GTK_CTREE(tree), ctree_node, 1, p, m);
	for (ll = nx->nodes; ll; ll = ll->next)
	  {
	     Node *nd;
	     GtkCTreeNode *ctree_leaf;
	     
	     nd = ll->data;
	     nd->selected = 0;
	     /* add leaf node for node */
	     text[0] = ""; text[1] = "", text[2] = nd->name;
	     ctree_leaf = gtk_ctree_insert_node(GTK_CTREE(tree), ctree_node, NULL,
						text, 1, NULL, NULL, NULL,
						NULL, FALSE, TRUE);
	     gtk_pixmap_get(GTK_PIXMAP(pm_good), &p, &m);
	     gtk_ctree_node_set_pixmap(GTK_CTREE(tree), ctree_leaf, 1, p, m);
	     gtk_ctree_node_set_row_data(GTK_CTREE(tree), ctree_leaf, nd);
	  }
     }
   /* unfreeze */
   gtk_clist_thaw(GTK_CLIST(tree));
}
