
/*
 * This program is original work by San Mehat <nettwerk@valinux.com> .
 * This program is distributed under the GNU General Public License (GPL)
 * as outlined in the COPYING file.
 *
 * Copyright (C) 1999, San Mehat, and VA Linux Systems
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 */

#include <config.h>

#include <stdio.h>
#include <sys/socket.h>
#include <signal.h>
#include <unistd.h>
#include <netinet/in.h>
#include <sys/un.h>
#include <stdlib.h>
#include <sys/time.h>
#include <dlfcn.h>
#include <sys/types.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <ctype.h>
#include <crypt.h>
#include <nexxus.h>
#include <module_harness.h>
#include <ipc_connection.h>
#include <prototypes.h>
#include <config.h>
extern VACM_MODULE_LIST *mlist_head;
extern VACM_MODULE_LIST *mlist_tail;
extern NEXXUS_USER			*user_head;
extern NEXXUS_USER			*user_tail;
extern NODEGROUP				*nodegroup_head;
extern NODEGROUP				*nodegroup_tail;

extern int                nexxus_api_version_major;
extern int                nexxus_api_version_minor;
extern int                nexxus_debug;

char *ipc_nexxus_debug(IPC_CONNECTION	*ipc,
												 NEXXUS_USER		*user,
												 NODE	*node,
												 __uint32_t			job_id,
												 char						**args)
{
  char message[64];

	KNOWN_UNUSED_PARAM(ipc);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(job_id);

  strncpy(message, args[0], (sizeof(message)-1));

  if(!(strcasecmp(message,"ON")))
    {
    nexxus_debug = 1;
	  nexxus_log_printf(NULL,
	    "User %s turned on nexxus debug mode.\n",user);
	  return("JOB_COMPLETED");
    }
  else if (!(strcasecmp(message,"OFF")))
    {
    nexxus_debug = 0;
	  nexxus_log_printf(NULL,
	    "User %s turned off nexxus debug mode.\n",user);
	  return("JOB_COMPLETED");
    }
  return("JOB_ERROR:INVALID_ARGUMENT");
}

char *ipc_nexxus_modules(IPC_CONNECTION	*ipc,
												 NEXXUS_USER		*user,
												 NODE	*node,
												 __uint32_t			job_id,
												 char						**args)
{
  char              msg[512];
  VACM_MODULE_LIST  *mlist = mlist_head;

	KNOWN_UNUSED_PARAM(user);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(args);

  while(mlist)
    {
    snprintf(msg,
             sizeof(msg),
             "NEXXUS:%d:MODULE:%s:%s:%s:%s:%d.%d:%s:%s",
              job_id,
              mlist->short_name,
              mlist->long_name,
              mlist->author,
              mlist->desc,
              mlist->major_version,
              mlist->minor_version,
              mlist->ipc_tag,
              return_module_state(mlist));
		if (send_socket_ipc(ipc, msg, ((int)strlen(msg)+1))<0)
			return(NULL);
    mlist = mlist->next;
    }
	return("JOB_COMPLETED");
}

char *ipc_node_add(IPC_CONNECTION	*ipc,
									 NEXXUS_USER		*user,
									 NODE	*node,
									 __uint32_t			job_id,
									 char						**args)
{
	NODEGROUP       *group;
  NODE  *new_node;
  char            node_name[NODE_NAME_MAXLENGTH];
  char            group_name[GROUP_NAME_MAXLENGTH];
 	char						msg[255];
   
	KNOWN_UNUSED_PARAM(ipc);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(job_id);

  strncpy(node_name, args[0], (sizeof(node_name)-1));
  strncpy(group_name, args[1], (sizeof(group_name)-1));

  /* See if this node name exists in *any* groups */
  if (nodegroup_node_lookup(NULL,node_name))
		return("JOB_ERROR:NODE_EXISTS");

  /* Get the group */
	if (!user_group_user_is_member(user, group_name))
		{
		nexxus_log_printf(NULL,
													"User %s failed to add node %s to group %s (bad group)\n",
													user->user_name,
													node_name,
													group_name);
		return("JOB_ERROR:BAD_GROUP");
		}

  if (!(group = nodegroup_lookup_by_name(group_name)))
		{
		group = nodegroup_create(group_name);
		user_group_member_add(user, group->group_name);
		}

	/* Link the node into the group */
  new_node = nodelist_entry_create(node_name);
  nodegroup_add_node(group, new_node);
  configuration_save();
	nexxus_log_printf(NULL,
												"User %s sucessfully added node %s to group %s\n",
												user->user_name,
												new_node->node_id,
												group->group_name);

  sprintf(msg, "ADDITION:%s",node_name);
  dispatch_config_line(NULL, msg);
	return("JOB_COMPLETED");
}

char *ipc_node_del(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
	char	node_id[NODE_NAME_MAXLENGTH];
	char	msg[255];

	KNOWN_UNUSED_PARAM(job_id);
	KNOWN_UNUSED_PARAM(args);

	strcpy(node_id, node->node_id);
  if (nodelist_entry_delete(ipc->user, node->node_id)<0)
		return("JOB_ERROR:NODE_NOT_FOUND");

  configuration_save();
	nexxus_log_printf(NULL,
												"User %s sucessfully deleted node %s\n",
												user->user_name,
												node_id);

  sprintf(msg, "DELETION:%s",node_id);
  dispatch_config_line(NULL, msg);
	return("JOB_COMPLETED");
}

char *ipc_module_unload(IPC_CONNECTION	*ipc,
												NEXXUS_USER			*user,
												NODE						*node,
												__uint32_t			job_id,
												char						**args)
{
	char	module[255];
	char	ipc_tag[255];
	KNOWN_UNUSED_PARAM(ipc);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(user);
	KNOWN_UNUSED_PARAM(job_id);

	strcpy(module, args[0]);
  if (!lookup_ipc_tag(module, ipc_tag))
		return("JOB_ERROR:MODULE_NOT_FOUND");

	shutdown_module(ipc_tag);
	return("JOB_COMPLETED");
}

char *ipc_module_load(IPC_CONNECTION	*ipc,
											NEXXUS_USER			*user,
											NODE						*node,
											__uint32_t			job_id,
											char						**args)
{
	char				filename[255];

	KNOWN_UNUSED_PARAM(ipc);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(user);
	KNOWN_UNUSED_PARAM(job_id);

	sprintf(filename,PKGLIBDIR "/modules/%s",args[0]);
	if (access(filename, R_OK|X_OK) <0)
		return("JOB_ERROR:UNABLE_TO_START_MODULE");

	start_loose_module(filename);
	return("JOB_COMPLETED");
}
char *ipc_node_rename(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
  char  old_name[NODE_NAME_MAXLENGTH];
  char  new_name[NODE_NAME_MAXLENGTH];
	char	msg[255];
  int   rc;
    
	KNOWN_UNUSED_PARAM(user);
	KNOWN_UNUSED_PARAM(job_id);

	strcpy(old_name, node->node_id);
	strcpy(new_name, args[0]);

  if ((rc = nodelist_entry_rename(ipc->user, old_name, new_name))<0)
    {
    if (rc == -1)
			return("JOB_ERROR:NODE_EXISTS");
   else if (rc == -2)
			return("JOB_ERROR:NODE_NOT_FOUND");
    }
  configuration_save();

  sprintf(msg, "RENAME:%s:%s",old_name, new_name);
  dispatch_config_line(NULL, msg);
	return("JOB_COMPLETED");
}

char *ipc_node_list(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
	NODEGROUP				*node_grp;
	USER_GROUP			*user_grp;
	NODE	*list_node;
	char						msg[255];

	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(args);

  user_grp = user->groups;
  while(user_grp)
    {
    if ((node_grp = nodegroup_lookup_by_name(user_grp->group_name)))
      {
      list_node = node_grp->nodelist;
      while(list_node)
        {
        sprintf(msg,
                "NEXXUS:%d:NODELIST:%s:%s",
                job_id,
                node_grp->group_name,
                list_node->node_id);
        if (send_socket_ipc(ipc,msg, (strlen(msg)+1))<0)
					return(NULL);
        list_node = list_node->next;
        }
      }
    user_grp = user_grp->next;
    }

	return("JOB_COMPLETED");
}

char *ipc_node_var_list(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
  NODEGROUP       *node_grp;
  USER_GROUP      *user_grp;
  NODE  *var_node;
  char            msg[255];
  char            varbuffer[255];

	KNOWN_UNUSED_PARAM(node);

  user_grp = user->groups;
  while(user_grp)
    {
    if ((node_grp = nodegroup_lookup_by_name(user_grp->group_name)))
      {
      var_node = node_grp->nodelist;
      while(var_node)
        {
        if (nexxus_variable_get(var_node->node_id,
                             NULL,
                             args[0],
                             varbuffer,
                             sizeof(varbuffer))<0)
          {
          *varbuffer = '\0';
          }
        sprintf(msg,
                "NEXXUS:%d:NODEVARLIST:%s:%s:%s",
                job_id,
                var_node->node_id,
                args[0],
                varbuffer);
        if (send_socket_ipc(ipc,msg, (strlen(msg)+1))<0)
					return(NULL);
        var_node = var_node->next;
        }
      }
    user_grp = user_grp->next;
    }

	return("JOB_COMPLETED");
}

char *ipc_get_var(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
  char  varname[64];
  char  varbuffer[255];
  char	msg[255];

	KNOWN_UNUSED_PARAM(user);

	strcpy(varname, args[0]);
  if (nexxus_variable_get(node->node_id, NULL, varname, varbuffer, sizeof(varbuffer))<0)
		return("JOB_ERROR:VAR_NOT_SET");

  sprintf(msg, "NEXXUS:%d:GET_VAR:%s",job_id, varbuffer);
  if (send_socket_ipc(ipc, msg, ((int)strlen(msg)+1))<0)
		return(NULL);

	return("JOB_COMPLETED");
}

char *ipc_set_var(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
  char  varname[64];
  char  varvalue[64];
  
	KNOWN_UNUSED_PARAM(ipc);
	KNOWN_UNUSED_PARAM(user);
	KNOWN_UNUSED_PARAM(job_id);

	strcpy(varname,args[0]);
	strcpy(varvalue,args[1]);

  if (nexxus_variable_set(node->node_id, NULL, varname, varvalue)<0)
		return("JOB_ERROR:NODE_NOT_FOUND");

	return("JOB_COMPLETED");
}

char *ipc_version(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
	char	msg[255];

	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(user);
	KNOWN_UNUSED_PARAM(args);

	sprintf(msg,
          "NEXXUS:%d:VERSION:%s:%d.%d",
          job_id,
          VERSION,
          nexxus_api_version_major,
          nexxus_api_version_minor);
  if (send_socket_ipc(ipc, msg, ((int)strlen(msg)+1))<0)
		return(NULL);

	return("JOB_COMPLETED");
}

char *ipc_auth(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
	char  username[64];
  char  password[64];
    
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(user);
	KNOWN_UNUSED_PARAM(job_id);

	strncpy(username,args[0],(sizeof(username)-1));
  strncpy(password,args[1],(sizeof(password)-1));

  if (ipc->user)
    {
#if 1 // BUG 168 fix
    nexxus_log_printf(NULL,
                          "Logging user %s out on client %d from %s\n",
                          ipc->user->user_name,
                          ipc->I_fd,
                          inet_ntoa(ipc->conn_address));
#endif
    ipc->user = NULL;     
    }
  if (!(authenticate_user(username, password)))
    {
    nexxus_log_printf(NULL,
                          "Auth failed for user %s in on client %d (bad auth)\n",
                          username,
                          ipc->I_fd);
		return("JOB_ERROR:BAD_AUTH");
    }
  /* Check to see if this user is allowed to log in from the source address */
  if (!verify_address_permissions((user_lookup(username)), &ipc->conn_address))
    {
    nexxus_log_printf(NULL,
            "Connection refused for user %s from address %s (fd %d)\n",
            username,
            inet_ntoa(ipc->conn_address),
            ipc->I_fd);
		return("JOB_ERROR:BAD_AUTH");
    }
  ipc->user = user_lookup(username);
#if 1 // BUG 168 fix
  nexxus_log_printf(NULL,
                        "Logging user %s in on client %d from %s\n",
                        ipc->user->user_name,
                        ipc->I_fd,
                        inet_ntoa(ipc->conn_address));
#endif
	return("JOB_COMPLETED");
}

char *ipc_echo(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
	char	msg[255];

	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(user);
	KNOWN_UNUSED_PARAM(job_id);

	sprintf(msg,"%s",args[0]);
	send_socket_ipc(ipc, msg, ((int)strlen(msg)+1));
	return(NULL); /* Echo doesnt send back a 'JOB COMPLETED' message */
}

char *ipc_admin_add(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
	NEXXUS_USER *new_user;
  NODEGROUP   *node_group;
  char        admin_name[255];
  char        passwd[42];
  char        group_name[255];
  int         i;
  
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(job_id);

  strncpy(admin_name, args[0], (sizeof(admin_name)-1));
	strncpy(passwd, args[1], (sizeof(passwd)-1));
  strncpy(group_name, args[2] ,(sizeof(group_name)-1));

  if (user_exists(admin_name))
		return("JOB_ERROR:USER_EXISTS");
  if (((int)strlen(passwd)) > 40)
		return("JOB_ERROR:PASSWORD_TOO_LONG");

  for (i=0;i<((int)strlen(passwd));i++)
    if (!isprint((int) passwd[i]))
			return("JOB_ERROR:BAD_PASSWORD");

  if (!(user_group_user_is_member(ipc->user, group_name)))
    {
    nexxus_log_printf(NULL,
                          "User %s permission denied to create user '%s' (not in same group)\n",
                          ipc->user,
                          group_name);
		return("JOB_ERROR:BAD_GROUP");
    }
  if (!(node_group = nodegroup_lookup_by_name(group_name)))
    node_group = nodegroup_create(group_name);
    
  new_user = user_create(admin_name);
  user_group_member_add(new_user, group_name);
  strcpy(new_user->password, crypt(passwd,"HI"));
  new_user->default_address_acl_policy = POLICY_DENY;
  new_user->default_module_acl_policy = POLICY_DENY;
  configuration_save();
  nexxus_log_printf(NULL,
                        "Admin %s sucessfully created new admin acount %s (grp %s)\n",
                        user->user_name, 
                        new_user->user_name,
                        group_name);

	return("JOB_COMPLETED");
}

char *ipc_admin_del(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
	NEXXUS_USER	*deleted_user;
  char				msg[255];

	KNOWN_UNUSED_PARAM(node);

  if (!(deleted_user = user_lookup(args[0])))
		return("JOB_ERROR:USER_NOT_FOUND");
  nexxus_log_printf(NULL,
                        "Admin %s sucessfully deleted admin acount %s\n",
                        user->user_name, 
                        deleted_user->user_name);

  sprintf(msg, "NEXXUS:%d:JOB_COMPLETED", job_id);
  send_socket_ipc(ipc, msg, ((int)strlen(msg)+1));
  if (ipc->user == deleted_user)
    destroy_ipc_connection(ipc);
  /* The actual deletion is delayed cuz destroy_ipc_connection needs user */
  user_destroy(deleted_user);

  configuration_save();
  return(NULL);
}

char *ipc_admin_rename(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
	NEXXUS_USER *rename_user;
  
	KNOWN_UNUSED_PARAM(ipc);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(job_id);

  if (!(rename_user = user_lookup(args[0])))
		return("JOB_ERROR:USER_NOT_FOUND");
  if (user_exists(args[1]))
		return("JOB_ERROR:NAME_IN_USE");

  strcpy(rename_user->user_name, args[1]);
  configuration_save();
  nexxus_log_printf(NULL,
                        "Admin %s sucessfully renamed admin %s to %s\n",
                        user->user_name, 
                        args[0],
                        args[1]);

	return("JOB_COMPLETED");
}

char *ipc_admin_list(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
	NEXXUS_USER *list_user;
  char				msg[255];

	KNOWN_UNUSED_PARAM(user);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(args);

  list_user = user_head;
  while(list_user)
    {
    sprintf(msg, "NEXXUS:%d:ADMIN_LIST:%s",job_id,list_user->user_name);
    if (send_socket_ipc(ipc, msg, ((int)strlen(msg)+1))<0)
			return(NULL);
    list_user = list_user->next;
    }

	return("JOB_COMPLETED");
}

char *ipc_admin_list_groups(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
	NEXXUS_USER	*list_user;
	NODEGROUP		*group;
	char				msg[255];

	KNOWN_UNUSED_PARAM(user);
	KNOWN_UNUSED_PARAM(node);

	if (!(list_user = user_lookup(args[0])))
		return("JOB_ERROR:USER_NOT_FOUND");

	group = nodegroup_head;
	while(group)
		{
		if (user_group_user_is_member(list_user, group->group_name))
			{
    	sprintf(msg, "NEXXUS:%d:ADMIN_LIST_GROUPS:%s",job_id,group->group_name);
    	if (send_socket_ipc(ipc, msg, ((int)strlen(msg)+1))<0)
				return(NULL);
			}
		group = group->next;
		}
	return("JOB_COMPLETED");
}

char *ipc_admin_add_mod_acl_rule(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
  NEXXUS_USER     *acl_user;
  USER_MODULE_ACL *mod_acl;
  int             I_policy;
  
	KNOWN_UNUSED_PARAM(ipc);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(job_id);

  if (!strcasecmp(args[1],"ALLOW"))
    I_policy = POLICY_ALLOW;
  else if (!strcasecmp(args[1], "DENY"))
    I_policy = POLICY_DENY;
  else
		return("JOB_ERROR:MALFORMATTED_MESSAGE");

  if (!(acl_user = user_lookup(args[0])))
		return("JOB_ERROR:USER_NOT_FOUND");

  if (user_module_acl_rule_lookup(acl_user, args[2], I_policy, args[3]))
		return("JOB_ERROR:DUPLICATE_RULE");

  if (!(mod_acl = user_module_acl_lookup(acl_user, args[2])))
    mod_acl = user_module_acl_create(acl_user, args[2]);
  user_module_acl_rule_create(mod_acl,I_policy, args[3]);
  configuration_save();
  
  nexxus_log_printf(NULL,
      "Admin %s sucessfully added module ACL rule to admin %s [%s]\n",
      user->user_name, 
      args[0],
      args[3]);

	return("JOB_COMPLETED");
}

char *ipc_admin_del_mod_acl_rule(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
  NEXXUS_USER     *acl_user;
  USER_MODULE_ACL *mod_acl;
  MODULE_RULE     *rule;
  int             I_policy;
  
	KNOWN_UNUSED_PARAM(ipc);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(job_id);
  
  if (!strcasecmp(args[1],"ALLOW"))
    I_policy = POLICY_ALLOW;
  else if (!strcasecmp(args[1], "DENY"))
    I_policy = POLICY_DENY;
  else
		return("JOB_ERROR:MALFORMATTED_MESSAGE");

  if (!(acl_user = user_lookup(args[0])))
		return("JOB_ERROR:USER_NOT_FOUND");

  if (!(rule =user_module_acl_rule_lookup(acl_user, args[2], I_policy, args[3])))
		return("JOB_ERROR:RULE_NOT_FOUND");

  /* Just a little sanity check here */
  if (!(mod_acl = user_module_acl_lookup(acl_user, args[2])))
		return("JOB_ERROR:INTERNAL_ERROR");
    
  user_module_acl_rule_destroy(mod_acl,rule);
  configuration_save();
  nexxus_log_printf(NULL,
      "Admin %s sucessfully deleted module ACL rule from admin %s [%s]\n",
      user->user_name, 
      args[0],
      args[3]);

	return("JOB_COMPLETED");
}

char *ipc_admin_list_mod_acl_rules(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
  NEXXUS_USER     *acl_user;
  USER_MODULE_ACL *acl;
  MODULE_RULE     *rule;
  char						msg[255];

	KNOWN_UNUSED_PARAM(user);
	KNOWN_UNUSED_PARAM(node);

  if (!(acl_user = user_lookup(args[0])))
		return("JOB_ERROR:USER_NOT_FOUND");
  acl = acl_user->mod_acl;
  while(acl)
    {
    rule = acl->rules;
    while(rule)
      {
      sprintf(msg,
              "NEXXUS:%d:MOD_ACL_RULE:%s:%s:%s",
              job_id,
              acl->module_ipc_tag,
              (rule->policy == POLICY_ALLOW ? "ALLOW" : "DENY"),
              rule->ipc_glob); 
      if (send_socket_ipc(ipc, msg, ((int)strlen(msg)+1))<0)
				return(NULL);
      rule = rule->next;
      }
    acl = acl->next;
    }

	return("JOB_COMPLETED");
}

char *ipc_admin_add_addr_acl_rule(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
  NEXXUS_USER       *acl_user;
  int               I_policy;
  int               I_sigbits;
  struct in_addr    addr;
  
	KNOWN_UNUSED_PARAM(ipc);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(job_id);

  if (!strcasecmp(args[1],"ALLOW"))
    I_policy = POLICY_ALLOW;
  else if (!strcasecmp(args[1], "DENY"))
    I_policy = POLICY_DENY;
  else
		return("JOB_ERROR:MALFORMATTED_MESSAGE");

  I_sigbits = atoi(args[3]);
  if (((I_sigbits == 0) && (args[3][0]!='0')) ||
       (I_sigbits <0) || (I_sigbits > 32))
		return("JOB_ERROR:BAD_SIGBITS");

  memset(&addr, 0, sizeof(addr));
  if (inet_aton(args[2], &addr))
		return("JOB_ERROR:BAD_ADDRESS");

  if (!(acl_user = user_lookup(args[0])))
		return("JOB_ERROR:USER_NOT_FOUND");

  if (user_address_acl_rule_lookup(acl_user, I_policy, &addr, I_sigbits))
		return("JOB_ERROR:DUPLICATE_RULE");

  user_address_acl_rule_create(acl_user, &addr, I_sigbits, I_policy);
  configuration_save();
  nexxus_log_printf(NULL,
      "Admin %s sucessfully added address ACL rule to admin %s\n",
      user->user_name, 
      args[0]);

	return("JOB_COMPLETED");
}

char *ipc_admin_del_addr_acl_rule(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
  NEXXUS_USER       *acl_user;
  USER_ADDRESS_RULE *rule;
  int               I_policy;
  int               I_sigbits;
  struct in_addr    addr;
  
	KNOWN_UNUSED_PARAM(ipc);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(job_id);

  if (!strcasecmp(args[1],"ALLOW"))
    I_policy = POLICY_ALLOW;
	else if (!strcasecmp(args[1], "DENY"))
    I_policy = POLICY_DENY;
  else
		return("JOB_ERROR:MALFORMATTED_MESSAGE");

  I_sigbits = atoi(args[3]);
  if (((I_sigbits == 0) && (args[3][0]!='0')) ||
       (I_sigbits <0) || (I_sigbits > 32))
		return("JOB_ERROR:BAD_SIGBITS");

  memset(&addr, 0, sizeof(addr));
  if (inet_aton(args[2], &addr))
		return("JOB_ERROR:BAD_ADDRESS");

  if (!(acl_user = user_lookup(args[0])))
		return("JOB_ERROR:USER_NOT_FOUND");

  if (!(rule=user_address_acl_rule_lookup(acl_user, I_policy, &addr, I_sigbits)))
		return("JOB_ERROR:RULE_NOT_FOUND");

  user_address_acl_rule_destroy(acl_user, rule);
  configuration_save();
  nexxus_log_printf(NULL,
      									"Admin %s sucessfully deleted address ACL rule to admin %s\n",
      									user->user_name, 
      									args[0]);

	return("JOB_COMPLETED");
}

char *ipc_admin_list_addr_acl_rules(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
  NEXXUS_USER       *acl_user;
  USER_ADDRESS_RULE *rule;
 	char							msg[255];
 
	KNOWN_UNUSED_PARAM(user);
	KNOWN_UNUSED_PARAM(node);

  if (!(acl_user = user_lookup(args[0])))
		return("JOB_ERROR:USER_NOT_FOUND");

  rule = acl_user->addr_acl;
  while(rule)
    {
    sprintf(msg,
            "NEXXUS:%d:ADDR_ACL_RULE:%s:%s:%d",
            job_id,
            (rule->policy == POLICY_ALLOW ? "ALLOW" : "DENY"),
            inet_ntoa(rule->ipv4_address),
            rule->ipv4_sigbits);
    if (send_socket_ipc(ipc, msg, ((int)strlen(msg)+1))<0)
			return(NULL);
    rule = rule->next;
    }

	return("JOB_COMPLETED");
}

char *ipc_admin_list_default_acl_policy(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
  NEXXUS_USER     *acl_user;
  int             policy;
  char						msg[255];

	KNOWN_UNUSED_PARAM(user);
	KNOWN_UNUSED_PARAM(node);

  if (!(acl_user = user_lookup(args[0])))
		return("JOB_ERROR:USER_NOT_FOUND");

  if (!strcasecmp(args[1],"MODULE"))
    policy = acl_user->default_module_acl_policy;
  else if (!strcasecmp(args[1],"ADDRESS"))
    policy = acl_user->default_address_acl_policy;
  else
		return("JOB_ERROR:BAD_ACL");
  sprintf(msg,
          "NEXXUS:%d:DEFAULT_POLICY:%s",
          job_id,
          (policy == POLICY_ALLOW ? "ALLOW" : "DENY"));
  if (send_socket_ipc(ipc, msg, ((int)strlen(msg)+1))<0)
		return(NULL);

	return("JOB_COMPLETED");
}

char *ipc_admin_chg_default_acl_policy(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
  NEXXUS_USER     *acl_user;
  int             I_policy;
  
	KNOWN_UNUSED_PARAM(ipc);
	KNOWN_UNUSED_PARAM(user);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(job_id);

  if (!(acl_user = user_lookup(args[0])))
		return("JOB_ERROR:USER_NOT_FOUND");

  if (!strcasecmp(args[2],"ALLOW"))
    I_policy = POLICY_ALLOW;
  else if (!strcasecmp(args[2], "DENY"))
    I_policy = POLICY_DENY;
  else
		return("JOB_ERROR:BAD_POLICY");
    
  if (!strcasecmp(args[1],"MODULE"))
    acl_user->default_module_acl_policy = I_policy;
  else if (!strcasecmp(args[1],"ADDRESS"))
    acl_user->default_address_acl_policy = I_policy;
	else
		return("JOB_ERROR:BAD_ACL");

  configuration_save();

	return("JOB_COMPLETED");
}

char *ipc_admin_chg_password(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
	KNOWN_UNUSED_PARAM(ipc);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(job_id);

  if (!authenticate_user(user->user_name, args[0]))
		return("JOB_ERROR:BAD_AUTH");

  strcpy(user->password, crypt(args[1], "HI"));
  configuration_save();
  
  nexxus_log_printf(NULL,
      									"Admin %s sucessfully changed password\n",
      									user->user_name);

	return("JOB_COMPLETED");
}

char *ipc_admin_set_password(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
	NEXXUS_USER *passwd_user;
  
	KNOWN_UNUSED_PARAM(ipc);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(job_id);

  if (!(passwd_user = user_lookup(args[0])))
		return("JOB_ERROR:USER_NOT_FOUND");
  strcpy(passwd_user->password, crypt(args[1], "HI"));
  configuration_save();
  
  nexxus_log_printf(NULL,
      "Admin %s sucessfully set password for admin %s\n",
      user->user_name, 
      passwd_user->user_name);

	return("JOB_COMPLETED");
}

char *ipc_group_add(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
	NODEGROUP	*group;

	KNOWN_UNUSED_PARAM(ipc);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(job_id);

	if ((group = nodegroup_lookup_by_name(args[0])))
		return("JOB_ERROR:GROUP_EXISTS");

	group = nodegroup_create(args[0]);
	configuration_save();
  nexxus_log_printf(NULL,
      "Admin %s sucessfully created group %s\n",
      user->user_name, 
      group->group_name);
	return("JOB_COMPLETED");
}

char *ipc_group_del(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
	NODEGROUP		*group;
	NEXXUS_USER	*user_scan;

	KNOWN_UNUSED_PARAM(ipc);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(job_id);

	if (!(group = nodegroup_lookup_by_name(args[0])))
		return("JOB_ERROR:GROUP_NOT_FOUND");
	if (!strcasecmp(group->group_name, "default"))
		return("JOB_ERROR:BAD_GROUP");
	if (group->nodelist)
		return("JOB_ERROR:GROUP_HAS_NODES");
	user_scan = user_head;
	while(user_scan)
		{
		if (user_group_user_is_member(user_scan, group->group_name))
			return("JOB_ERROR:GROUP_HAS_MEMBERS");
		user_scan = user_scan->next;
		}
	
	nodegroup_destroy(group);
	configuration_save();
  nexxus_log_printf(NULL,
      									"Admin %s sucessfully deleted group %s\n",
      									user->user_name, 
      									group->group_name);
	return("JOB_COMPLETED");
}

char *ipc_group_list(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
	NODEGROUP	*group_scan;
	char			msg[255];

	KNOWN_UNUSED_PARAM(user);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(job_id);
	KNOWN_UNUSED_PARAM(args);

	group_scan = nodegroup_head;
	while(group_scan)
		{
		sprintf(msg,
						"NEXXUS:%d:GROUP_LIST:%s",
						job_id,
						group_scan->group_name);
  	if (send_socket_ipc(ipc, msg, ((int)strlen(msg)+1))<0)
			return(NULL);
		group_scan = group_scan->next;
		}
	return("JOB_COMPLETED");
}

char *ipc_group_rename(IPC_CONNECTION	*ipc,
											 NEXXUS_USER		*user,
											 NODE	*node,
											 __uint32_t			job_id,
											 char						**args)
{
	NODEGROUP		*group;
	NEXXUS_USER	*rename_user;
	USER_GROUP	*user_group;

	KNOWN_UNUSED_PARAM(ipc);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(job_id);

	if (!(group = nodegroup_lookup_by_name(args[0])))
		return("JOB_ERROR:GROUP_NOT_FOUND");
	if (nodegroup_lookup_by_name(args[1]))
		return("JOB_ERROR:GROUP_EXISTS");
	strncpy(group->group_name, args[1], (sizeof(group->group_name)-1));
	/* Now find any users that were members and rename the group */
	rename_user = user_head;
	while(rename_user)
		{
		if ((user_group = user_group_user_is_member(rename_user, args[0])))
			strncpy(user_group->group_name,args[1],(sizeof(user_group->group_name)-1));
		rename_user = rename_user->next;
		}
	configuration_save();
  nexxus_log_printf(NULL,
      									"Admin %s sucessfully renamed group %s to %s\n",
      									user->user_name, 
      									args[0],
												args[1]);
	return("JOB_COMPLETED");
}

char *ipc_group_add_admin(IPC_CONNECTION *ipc, NEXXUS_USER *user, NODE *node, __uint32_t job_id, char **args)
{
	NEXXUS_USER	*group_user;
	NODEGROUP		*group;

	KNOWN_UNUSED_PARAM(ipc);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(job_id);

	if (!(group = nodegroup_lookup_by_name(args[0])))
		return("JOB_ERROR:GROUP_NOT_FOUND");
	if (!(group_user = user_lookup(args[1])))
		return("JOB_ERROR:USER_NOT_FOUND");
	if (user_group_user_is_member(group_user, group->group_name))
		return("JOB_ERROR:ALREADY_IN_GROUP");
	user_group_member_add(group_user, group->group_name);
	configuration_save();
  nexxus_log_printf(NULL,
      									"Admin %s sucessfully added user %s to group %s\n",
      									user->user_name, 
      									group_user->user_name,
												group->group_name);
	return("JOB_COMPLETED");
}

char *ipc_group_del_admin(IPC_CONNECTION	*ipc,
													NEXXUS_USER			*user,
													NODE	*node,
													__uint32_t			job_id,
													char						**args)
{
	NEXXUS_USER	*group_user;
	NODEGROUP		*group;

	KNOWN_UNUSED_PARAM(ipc);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(job_id);

	if (!(group = nodegroup_lookup_by_name(args[0])))
		return("JOB_ERROR:GROUP_NOT_FOUND");
	if (!(group_user = user_lookup(args[1])))
		return("JOB_ERROR:USER_NOT_FOUND");
	if (!user_group_user_is_member(group_user, group->group_name))
		return("JOB_ERROR:NOT_IN_GROUP");
	user_group_member_remove(group_user, group->group_name);
	configuration_save();
  nexxus_log_printf(NULL,
      									"Admin %s sucessfully removed user %s from group %s\n",
      									user->user_name, 
      									group_user->user_name,
												group->group_name);
	return("JOB_COMPLETED");
}

char *ipc_node_set_group(IPC_CONNECTION	*ipc,
												 NEXXUS_USER		*user,
												 NODE	*node,
												 __uint32_t			job_id,
												 char						**args)
{
	NODEGROUP	*old_group;
	NODEGROUP	*new_group;

	KNOWN_UNUSED_PARAM(ipc);
	KNOWN_UNUSED_PARAM(user);
	KNOWN_UNUSED_PARAM(job_id);

	if (!(new_group = nodegroup_lookup_by_name(args[0])))
		return("JOB_ERROR:GROUP_NOT_FOUND");
	
	/* Remove the node from the old group */
	if (!(old_group = nodegroup_lookup_by_node(node)))
		{
		nexxus_log_printf(NULL,
													"Warning: Node %s doesn't have a group\n",
													node->node_id);
		}
	else
		{
		if (old_group == new_group)
			return("JOB_ERROR:ALREADY_IN_GROUP");
		nodegroup_remove_node(old_group, node);
		}
	nodegroup_add_node(new_group, node);
	configuration_save();
	return("JOB_COMPLETED");
}

char *ipc_set_chatty(IPC_CONNECTION *ipc,NEXXUS_USER *user,NODE	*node,
		    __uint32_t job_id,char **args)
{
	KNOWN_UNUSED_PARAM(user);
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(job_id);

	ipc->chatty = (!strcasecmp(args[0],"off")) ? 0 : 1;
	return("JOB_COMPLETED");
}


char *ipc_list_clients(IPC_CONNECTION	*ipc,
							 NEXXUS_USER		*user,
							 NODE						*node,
							 __uint32_t			job_id,
							 char						**args)
{
	IPC_CONNECTION	*ipc_scan;
	char			      msg[255];
  time_t          curr_time = time(NULL);

  KNOWN_UNUSED_PARAM(user);
  KNOWN_UNUSED_PARAM(node);
  KNOWN_UNUSED_PARAM(args);

  ipc_scan = ipc;
  /* Scan to the beginning of the list */
  while(ipc_scan->prev)
    ipc_scan = ipc_scan->prev;

  while(ipc_scan)
    {
    long int idle_seconds = curr_time - ipc_scan->idle_timer;
		snprintf(msg,
            sizeof(msg),
						"NEXXUS:%d:LIST_CLIENTS:%d:%s:%ld",
						job_id,
            ipc_scan->I_fd,
						(char *)ipc_scan->user,
            idle_seconds);
  	if (send_socket_ipc(ipc, msg, ((int)strlen(msg)+1))<0)
			return(NULL);
		ipc_scan = ipc_scan->next;
    }
  return("JOB_COMPLETED");
}

char *ipc_wall(IPC_CONNECTION	*ipc,
							 NEXXUS_USER		*user,
							 NODE						*node,
							 __uint32_t			job_id,
							 char						**args)
{
	char	wall_msg[1024];
	KNOWN_UNUSED_PARAM(node);
	KNOWN_UNUSED_PARAM(job_id);

	snprintf(wall_msg,(sizeof(wall_msg)-1),"NEXXUS:0:WALL:%s:%s:%s",
					user->user_name,
					inet_ntoa(ipc->conn_address),
					args[0]);
	if (strlen(wall_msg) > 512)
		return("JOB_ERROR:MSG_TOO_LONG");

	send_msg_to_all_clients(wall_msg, (strlen(wall_msg)+1));
	return("JOB_COMPLETED");
}
