/*
 * This program is original work by San Mehat <nettwerk@valinux.com> .
 * This program is distributed under the GNU General Public License (GPL)
 * as outlined in the COPYING file.
 *
 * Copyright (C) 1999, San Mehat, and VA Linux Systems
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/un.h>
#include <sys/time.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <ctype.h>
#include <math.h>
#include <pthread.h>
#include <errno.h>
#include <termios.h>
#include <sys/ioctl.h>
#ifndef __sgi
#include <linux/types.h>
#include <linux/serial.h>
#endif
#include <libloose.h>
#include <emp_module_harness.h>
#include <emp_slave_context.h>
#include <prototypes.h>
#include <emp_slave_context.h>
#include <pthread.h>
#include <iana_list.h>
#include <emp.h>
#include <emp_sdr.h>
#include <emp_sel.h>
#include <emp_sensor_events.h>

int unnormalize_data(unsigned char *src, unsigned char *dst,int I_srclen)
{
  int I_srcpos,I_dstpos;

  for (I_srcpos=0,I_dstpos=0;I_srcpos<I_srclen;I_srcpos++)
    {
    if (src[I_srcpos]== (unsigned char) EMP_DATA_ESCAPE_B0)
      {
      dst[I_dstpos++]= (unsigned char) EMP_DATA_ESCAPE_CHAR;
      dst[I_dstpos++]= (unsigned char) 0xb0;
      }
    else if (src[I_srcpos]== (unsigned char) EMP_DATA_ESCAPE_B5)
      {
      dst[I_dstpos++]= (unsigned char) EMP_DATA_ESCAPE_CHAR;
      dst[I_dstpos++]= (unsigned char) 0xb5;
      }
    else if (src[I_srcpos]== (unsigned char) EMP_DATA_ESCAPE_BA)
      {
      dst[I_dstpos++]= (unsigned char) EMP_DATA_ESCAPE_CHAR;
      dst[I_dstpos++]= (unsigned char) 0xba;
      }
    else if (src[I_srcpos]== (unsigned char) EMP_DATA_ESCAPE_B8)
      {
      dst[I_dstpos++]= (unsigned char) EMP_DATA_ESCAPE_CHAR;
      dst[I_dstpos++]= (unsigned char) 0xb8;
      }
    else if (src[I_srcpos]== (unsigned char) EMP_DATA_ESCAPE_B6)
      {
      dst[I_dstpos++]= (unsigned char) EMP_DATA_ESCAPE_CHAR;
      dst[I_dstpos++]= (unsigned char) 0xb6;
      }
    else
      dst[I_dstpos++]=src[I_srcpos];
    }
  return(I_dstpos);
}

int normalize_data(unsigned char *src, unsigned char *dst)
{
  int I_c1,I_c2,I_pktlen;

  if (src[0]!=(unsigned char) 0xa0)
    return(-EFAULT);
  I_pktlen = 0;
  for (I_c1=1,I_c2=0;src[I_c1]!=(unsigned char) 0xa5;I_c1++)
    {
    if (I_c1>64)
      {
      return(-EFAULT);
      }
    if (src[I_c1]== (unsigned char) EMP_DATA_ESCAPE_CHAR)
      {
      dst[I_c2]=(src[I_c1+1] & ~0x10);
      I_c2++;
      I_c1++;
      }
    else
      {
      dst[I_c2]=src[I_c1];
      I_c2++;
      }
    }
  dst[I_c2]=src[I_c1];
  I_pktlen = I_c2;
  if (I_pktlen>EMP_MAX_MSG_LEN)
    {
    return(-EOVERFLOW);
    }
  return(I_pktlen);
}

char *return_ccstr(unsigned char cc)
{
  switch(cc)
    {
    case 0x0:
     return("Command Completed OK");
    case 0xc0:
      return("Node Busy");
    case 0xc1:
      return("Invalid Command");
    case 0xc2:
      return("Command Invalid for LUN");
    case 0xc3:
      return("Timeout");
    case 0xc4:
      return("Out of Space");
    case 0xc5:
      return("Reservation Canceled or Invalid Reservation ID");
    case 0xc6:
      return("Request data truncated");
    case 0xc7:
      return("Request data length invalid");
    case 0xc8:
      return("Request data field length limit exceeded");
    case 0xc9:
      return("Paramter out of range");
    case 0xca:
      return("Cannot return number of requested data bytes");
    case 0xcb:
      return("Requested sensor/data/record not present");
    case 0xcc:
      return("Invalid data field in request");
    case 0xcd:
      return("Command illegal for specified sensor / record");
    case 0xce:
      return("Command response could not be provided");
    case 0xcf:
      return("Cannot execute duplicated request");
    case 0xd0:
      return("SDR Repository in update mode");
    case 0xd1:
      return("Device in firmware update mode");
    case 0xd2:
      return("BMC init agent in progress");
    default:
      return("Unknown Condition Code");
    }
}
/*
 *  * Send a message. This function is threadsafe
 *   * SAN_FIX: 07/24/00 : Liar.. *now* it's threadsafe
 *    */

int send_msg(CONTEXT *ctx, int fd, char *msg)
{
  __uint32_t  len;
  __uint32_t  net_len;
  int         rc;
  
  if (fd == -1)
    return(-EBADF);
  pthread_mutex_lock(&ctx->ipc_writelock);
    
  len = strlen(msg)+1;
  net_len = htonl(len);
  if ((rc = write (fd, (void *) &net_len, sizeof(net_len))) < 0)
    {
    pthread_mutex_unlock(&ctx->ipc_writelock);
    return(rc);
    }
  if ((rc = write (fd, (void *) msg, len)) < 0)
    {
    pthread_mutex_unlock(&ctx->ipc_writelock);
    return(rc);
    }
  pthread_mutex_unlock(&ctx->ipc_writelock);
  return(0);
}   

int lookup_sensor_number(SENSOR *sensor)
{
	if (sensor->rec_header.record_type == 0x01)
		return(sensor->sensor_type.type_01_sensor.sensor_number);
	else if (sensor->rec_header.record_type == 0x02)
		return(sensor->sensor_type.type_02_sensor.sensor_number);
	else
		return(-1);
}

int recv_msg(int fd, char **buffer, __uint32_t *len)
{
  __uint32_t  net_len;
  __uint32_t  brtr;
  char        *offset;
  int         rc;

  if ((rc = read(fd, (void *) &net_len, sizeof(net_len))) < 0)
    return(rc);
  else if (!rc)
    return(-1);
  *len = ntohl(net_len);

  if (!(*buffer = malloc(*len)))
    return(-ENOMEM);
  memset(*buffer, 0, *len);
  brtr = *len;
  offset = *buffer;
  while(brtr)
    {
    if ((rc = read(fd, (void *) offset, brtr)) < 0)
      return(rc);
    else if (!rc)
      return(-1);
    brtr -= rc;
    offset += rc;
    }
  return(0);
}

