/*
 * This program is original work by San Mehat <nettwerk@valinux.com> and
 * Zac Sprackett <zac@sprackett.com>.
 * This program is distributed under the GNU Lesser General Public License
 * (LGPL).
 *
 * Copyright (C) 1999, San Mehat, Zac Sprackett and VA Linux Systems
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
#include <util.h>

void list_init(List *list, void (*destroy)(void *data))
{
	list->size = 0;
	list->destroy = destroy;
	list->head = NULL;
	list->tail = NULL;
}

void list_destroy(List *list)
{
	void *data;

	while(list_size(list)>0)
		{
		if ((!(list_rem_next(list,NULL,(void **)&data)))&&list->destroy != NULL)
			{
			/* The user passed in a destroy function */
			list->destroy(data);
			}
		}
	memset(list,0,sizeof(List));
}

int list_ins_next(List *list, List_Element *element, const void *data)
{
	List_Element *new_element;

	if ((new_element = (List_Element *)malloc(sizeof(List_Element)))==NULL)
		return(-1);

	new_element->data = (void *)data;

	if (element == NULL)
		{
		/* Insert at the head of the list */
		if (list_size(list) == 0)
			list->tail = new_element;

		new_element->next = list->head;
		list->head = new_element;
		}
	else
		{
		if (element->next == NULL)
			list->tail = new_element;

		new_element->next = element->next;
		element->next = new_element;
		}

	/* Adjust our list size */
	list->size++;
	return(0);
}

int list_rem_next(List *list, List_Element *element, void **data)
{
	List_Element *old_element;

	if(list_size(list) == 0)
		{
		/* The list is already empty */
		return(-1);
		}

	if(element == NULL)
		{
		/* Remove the head of the list */
		*data = list->head->data;

		old_element = list->head;
		list->head = list->head->next;
		list->size--;

		if(list_size(list) == 0)
			list->tail = NULL;
		}
	else
		{
		if(element->next == NULL)
			return(-1);

		*data = element->next->data;
		old_element = element->next;
		element->next = element->next->next;

		if(element->next == NULL)
			list->tail = element;

		list->size--;
		}

	free(old_element);
	return(0);
}

int list_rem(List *list, List_Element *element, void **data, List_Element **ptr)
{
	List_Element *old_element = NULL;

	if (list_size(list)==0)
		return(-1);

	if(element == list->head)
		{
		*data = list->head->data;

		old_element = list->head;
		list->head = list->head->next;
		list->size--;

		if(list_size(list) == 0)
			list->tail = NULL;
				
		if (ptr)
			*ptr = list->head;

		return(0);
		}
	else
		{
		/* Find our element */
		List_Element *tmp_scan = list->head;

		while(tmp_scan->next)
			{
			if (tmp_scan->next == element)
				{
				old_element = tmp_scan->next;
				tmp_scan->next = tmp_scan->next->next;

				if(tmp_scan->next == NULL)
					list->tail = element;

				list->size--;
				
				if (ptr)
					*ptr = tmp_scan->next;
				return(0);
				}
			tmp_scan = tmp_scan->next;
			}
		}
	return(-1);
}
